package ch.zhaw.projekt2.turnierverwaltung;

import javafx.application.Platform;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

public class TournamentDecorator implements IsObservable{
    private Tournament tournament;
    private FileIO fileIO;
    private List<IsObserver> listener = new ArrayList<>();
    private ExecutorService executorService;
    private FactoryDecorator factoryDecorator;

    public TournamentDecorator(FileIO fileIO){
        setFileIO(fileIO);
        addListener(new IsObserver() {
            @Override
            public void update() {
                if(tournament != null){

                    saveTournament();
                }
            }
        });
        executorService = Executors.newFixedThreadPool(Runtime.getRuntime().availableProcessors());
    }

    public void setFactoryDecorator(FactoryDecorator factoryDecorator) {
        this.factoryDecorator = factoryDecorator;
    }

    public void setFileIO(FileIO fileIO) {
        this.fileIO = fileIO;
    }

    public void setTournament(Tournament tournament) {
        this.tournament = tournament;
    }

    public Tournament getTournament() {
        return tournament;
    }

    @Override
    public void addListener(IsObserver observer) {
        listener.add(observer);
    }

    @Override
    public void removeListener(IsObserver observer) {
        listener.remove(observer);
    }


    public void saveTournament(){
        executorService.execute(new saveTask());
        factoryDecorator.clearMessage(false);
        factoryDecorator.printMessageToFooter("Zuletzt gespeichert: " + new Date().toString(),false);
    }



    public void createTournament(String name, Tournament.Type type){

        try {
            if(fileIO.tournamentExists(name)){
                //TODO:Logger
                factoryDecorator.printMessageToFooter("Turniername existiert bereits",true);
                return;
            }
            Tournament tournament = new Tournament(name, type);
            fileIO.saveTournament(tournament);
            factoryDecorator.clearMessage(true);
            informListener();
        } catch (InvalidNameException e) {
            e.printStackTrace();
            //TODO: Logger
            factoryDecorator.printMessageToFooter("Invalider Turniername",true);
        } catch (Tournament.InvalidTypeException e) {
            e.printStackTrace();
            //TODO: Logger
            factoryDecorator.printMessageToFooter("Turniermodus nicht möglich",true);

        } catch (IOException e) {
            e.printStackTrace();
            //TODO: Logger
            factoryDecorator.printMessageToFooter("Fehler bei Dateizugriff",true);
        }

    }

    public void deleteTournament(FileIO.TournamentFile tournamentFile){
        try {
            fileIO.deleteTournament(tournamentFile);
            factoryDecorator.clearMessage(true);
            informListener();
        } catch (IOException e) {
            e.printStackTrace();  //TODO handle and logging
            factoryDecorator.printMessageToFooter("Fehler bei Dateizugriff",true);
        }
    }

    public void createNewGameSchedule() {
        //TODO: logging
        try {
            tournament.createGameSchedule();
            factoryDecorator.clearMessage(true);
        } catch (Tournament.NumberOfParticipantInvalidException e) {
            e.printStackTrace();
            factoryDecorator.printMessageToFooter("Anzahl Teilnehmer nicht vielfaches von 2",true);
        }
        informListener();
    }

    public void savePlayer(String firstName, String name, String phoneNumber, String dateOfBirth){
        try {
            tournament.saveParticipant(new Player(firstName, name, phoneNumber, dateOfBirth));
            factoryDecorator.clearMessage(true);
            informListener();
        } catch (InvalidNameException e) {
            e.printStackTrace(); //TODO handle and logging
            factoryDecorator.printMessageToFooter("Invalider Teilnehmername",true);
        } catch (Person.InvalidPhoneNumberException e) {
            e.printStackTrace(); //TODO handle and logging
            factoryDecorator.printMessageToFooter("Invalide Telefonnummer",true);
        }
    }

    public void deleteParticipant(Participant participant){
        try {
            tournament.removeParticipant(participant);
            factoryDecorator.clearMessage(true);
            informListener();
        } catch (Tournament.ParticipantNotExistsException e) {
            e.printStackTrace(); //TODO handle and logging
            factoryDecorator.printMessageToFooter("Teilnehmer existiert nicht",true);

        }
    }

    public void savePlace(String name){
        try {
            tournament.addPlace(new Place(name));
            factoryDecorator.clearMessage(true);
            informListener();
        } catch (InvalidNameException e) {
            e.printStackTrace(); //TODO handle and logging
            factoryDecorator.printMessageToFooter("Invalider Ortsname",true);
        }
    }

    public void deletePlace(Place place){
        try {
            tournament.removePlace(place);
            factoryDecorator.clearMessage(true);
            informListener();
        } catch (Tournament.PlaceNotExistsException e) {
            e.printStackTrace(); //TODO handle and logging
            factoryDecorator.printMessageToFooter("Ort existiert nicht",true);
        }
    }


    public void informListener() {
        for(IsObserver observer : listener) {
            observer.update();
        }
    }

    private class saveTask implements Runnable {

        @Override
        public void run() {
            try {
                fileIO.saveTournament(tournament);
            } catch (IOException e) {
                e.printStackTrace();
                Platform.runLater(() -> {
                    factoryDecorator.printMessageToFooter("Datenzugriff nicht moeglich",true);
                });
            }
        }
    }

}
